## ------------------------------------------------
## Running a regression model in R
## + 2 interludes on variable scope and lists 
## ------------------------------------------------
## We are going to use the Burt dataset inside the car package:
## This is Fraudulent Data on IQs of Twins Raised Apart
install.packages("car")
library(car)
library(dplyr)
library(ggplot2)
library(gridExtra)

## -----------------------------------------------
## Environment and Variable Scope in R
## -----------------------------------------------

## The following variables exist in the global environment
global1 <- 7
global2 <- c(2, 9, 14)

## Let's write a function called my_function that takes in two arguments
## The first argument a is an integer 
## The second argument b is a vector
## The function adds a to each element in b and returns the resulting vector 



## Now let's call this function, passing it global1 and global2


## What happens when we try to access variables we declared in the function? 
a
b

## Now let's take a look at another function
colours <- c("red", "blue", "violet")
colour_to_add <- "yellow"

## What does this function do?
my_colour_function1 <- function(colours) {
  colours <- c(colours, colour_to_add)
  return(colours)
}

## What does our vector 'colours' look like after we call the function? Why?
my_colour_function1(colours)
colours # 'Colours' has not changed. We only changed the value of the local 'colours' vector

## What if we do the following? What does our vector 'colours' now look like? 
colours <- my_colour_function1(colours)
colours # Now we've assigned the global vector 'colours' the value of the return value of our function

## General suggestion: it's good to not duplicate variable names to avoid confusion! 

## -----------------------------------------------
## Lists in R
## -----------------------------------------------
## Create a list
n = c(2, 3, 5) 
s = c("aa", "bb", "cc", "dd", "ee") 
y = c(TRUE, FALSE, TRUE, FALSE, FALSE) 
mylist = list(n, s, y, 3)   # x contains copies of n, s, b
names(mylist) <- c("n", "s", "b", "constant")

## list slice - not a copy
mylist[1]
mylist$n
mylist[c(2, 4)]

## member reference - copy
mylist[[2]][1]

## -----------------------------------------------
## Regression
## -----------------------------------------------
head(Burt) # take a look at the data

## Simple OLS case of one independent variable

## create a simple plot
ggplot(data = Burt, aes(x = IQbio, y = IQfoster)) +
  geom_point(shape = 16, size = 3) +
  xlab("IQ, Raised by Biological Family") +
  ylab("IQ, Raised by Foster Family")


## Regress IQfoster on IQbio

## equation for the regression line
## Burt$IQfoster = 13.21931 + 0.86296 * Burt$IQbio




## Interpreting the results
## What is the null hypothesis?

## What are the test statistics?

## Can we reject the null at the 95% level?

## what can we extract from the lm output?


## multiple ways to extract the Intercept






## calculate test statistic by hand


## calculate pvalue by hand



## store intercept and slope




## Re-do scatterplot but add regression line
ggplot(data = Burt, aes(x = IQbio, y = IQfoster)) +
  geom_point(shape = 16, size = 3) +
  xlab("IQ, Raised by Biological Family") +
  ylab("IQ, Raised by Foster Family") +
  ggtitle("Nature vs. Nurture: \nThe IQ of Twins Raised in Separate Homes") +
  geom_abline(intercept = fit.int, slope = fit.slope, color = "blue")


## We can also have ggplot fit the line for us
## Let's check that the regression lines match
ggplot(data = Burt, aes(x = IQbio, y = IQfoster)) +
  geom_point(shape = 16, size = 3) +
  xlab("IQ, Raised by Biological Family") +
  ylab("IQ, Raised by Foster Family") +
  ggtitle("Nature vs. Nurture: \nThe IQ of Twins Raised in Separate Homes") +
  geom_abline(intercept = fit.int, slope = fit.slope, color = "blue") + 
  geom_smooth(method = "lm", se = FALSE, color = "red")



## let's look at the fitted values



## let's take a look at the residuals, 
## which R calculates as the actual values minus fitted values.


## let's calculate our own residuals



## check residuals we calculated are the same as those from lm()



## you can also sum the square of the residuals



## and calculate the R squared value



## look at the lm() output again



## let's add residual lines onto the plot
ggplot(data = Burt, aes(x = IQbio, y = IQfoster)) +
  geom_point(shape = 16, size = 3) +
  xlab("IQ, Raised by Biological Family") +
  ylab("IQ, Raised by Foster Family") +
  ggtitle("Nature vs. Nurture: \nThe IQ of Twins Raised in Separate Homes") +
  geom_abline(intercept = fit.int, slope = fit.slope, color = "blue") +
  geom_segment(aes(x = IQbio, xend = IQbio, y = IQfoster, yend = fit$fitted.values),
               color = "green")

## We can also create a plot of residuals 
## Does it look like we have homoskedasticity? Zero conditional mean?
ggplot(data = Burt, aes(x = fit$fitted.values, y = fit$residuals)) +
  geom_point(shape = 16, size = 3) + 
  xlab("Fitted Values") +
  ylab("Residuals") + 
  ggtitle("Fitted Values vs Residuals") + 
  geom_abline(intercept = 0, slope = 0, color = "blue") 

## ------------------------------------------------
## Using sampling
## ------------------------------------------------
## let's generate some data and pretend that it's the true population.
## we can do this with the rnorm function
set.seed(08544)
x <- rnorm(5000, mean = 7, sd = 1.56) # just some normally distributed data

## We're establishing here a linear relationship,
## What's this "true" linear relationship we're setting up? 
y <- 12 - 0.4*x + rnorm(5000, mean = 0, sd = 1)

## Let's  check out the histograms separately
qplot(x, data = data.frame(x))
qplot(y, data = data.frame(y))
## You know they are normal because??

## plotting them together:
ggplot(data = data.frame(x,y), aes(x = x, y = y)) +
  geom_point()
## you see they are roughly linear

## Our regression should give us back estimates for our
## intercept and slope (remember we're at the population level)



ggplot(data = data.frame(x,y), aes(x = x, y = y)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE, color = "coral")

## NOW let's sample from this the way we did in the univariate 
## statistics part of the course:




## we can set up a for loop to sample a thousand times,
## and store the regression coefficients from each run



## this could take a while depending
## on the number of simulations you're performing!

## So lets rewrite it using the replicate() function
## First we have to define the function replicate
# unlike most of our functions, we don't need any arguments!
my_simulation <- function() {
  my.samp <- sample_n(data, size)
  samp.lm <- lm(my.samp$y ~ my.samp$x)
  holder <- samp.lm$coefficients[1]
  holder[2] <- samp.lm$coefficients[2] 
  return(holder)
}


## Now we just tell it to do that 1000 times!
results <- replicate(1000, my_simulation())
## Our results come out with the coefficients on the 
## rows and simulations on the columns.  
## We will change this by transposing the matrix 
## to make it easier to look at.
holder <- data.frame(t(results))
colnames(holder) <- c("Intercept", "Slope")

## What do all these lines look like plotted together?
ggplot() +
  scale_x_continuous(name = "X Values", limits = c(0, max(x))) + 
  scale_y_continuous(name = "Y Values", limits = c(0, max(y))) + 
  geom_abline(data = holder, aes(intercept = Intercept, slope = Slope)) +
  ggtitle("Fitted regression lines from 1000 samples of size 1000")


## We can also plot the marginal distribution
## of the slope and intercept:

## as a side note, let's use Color Brewer to pick some pretty colors for us
## http://www.r-bloggers.com/r-using-rcolorbrewer-to-colour-your-figures-in-r/
library(RColorBrewer)
display.brewer.all()
graph.colors <- brewer.pal(8, "Pastel2") # pick a palette

hist1000.b0 <- ggplot(data = holder, aes(x = Intercept)) + 
  geom_histogram(breaks = seq(10, 14, by = 0.1), fill = graph.colors[6], color = graph.colors[8]) +
  ggtitle(expression("Sampling distribution for " * beta[0])) +
  xlab(expression(beta[0])) +
  xlim(10,14) + 
  geom_vline(xintercept = mean(holder[,1]), color = "blue")

hist1000.b1 <- ggplot(data = holder, aes(x = Slope)) + 
  geom_histogram(breaks = seq(-0.65, -0.25, by = 0.01), fill = graph.colors[4], color = graph.colors[8]) +
  ggtitle(expression("Sampling distribution for " * beta[1])) +
  xlab(expression(beta[1])) +
  xlim(-0.65,-0.25) +
  geom_vline(xintercept = mean(holder[,2]), color = "blue")

grid.arrange(hist1000.b0, hist1000.b1, nrow = 1, ncol = 2)


## two questions:
##	1. why do they look normal? (and why does that make sense?)
##	2. what's the mean of these two distributions? (and why does that make sense?)


